"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const puppeteer_extra_1 = __importDefault(require("puppeteer-extra"));
const puppeteer_extra_plugin_stealth_1 = __importDefault(require("puppeteer-extra-plugin-stealth"));
const axios_1 = __importDefault(require("axios"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const child_process_1 = __importDefault(require("child_process"));
const adm_zip_1 = __importDefault(require("adm-zip"));
puppeteer_extra_1.default.use((0, puppeteer_extra_plugin_stealth_1.default)());
class BedrockServer {
    constructor(folder) {
        this.folder = folder;
    }
    static getLatestDownloadURL(url = 'https://www.minecraft.net/en-us/download/server/bedrock') {
        return __awaiter(this, void 0, void 0, function* () {
            const browser = yield puppeteer_extra_1.default.launch({
                headless: false,
                args: ['--no-sandbox', '--disable-setuid-sandbox'],
                defaultViewport: {
                    width: 1920,
                    height: 1080
                }
            });
            const [page] = yield browser.pages();
            page.setDefaultNavigationTimeout(0);
            const session = yield page.target().createCDPSession();
            const { windowId } = yield session.send('Browser.getWindowForTarget');
            yield session.send('Browser.setWindowBounds', { windowId, bounds: { windowState: 'minimized' } });
            yield page.goto(url);
            const el = yield page.waitForSelector('#main-content > div > div > div.page-section-container.aem-GridColumn.aem-GridColumn--default--12 > div > div > div > div.server-card.aem-GridColumn.aem-GridColumn--default--12 > div > div > div > div:nth-child(1) > div.card-footer > div > a');
            const value = (yield (yield el.getProperty('href')).jsonValue()).split('/').pop();
            yield page.close();
            yield browser.close();
            const os = process.platform === 'win32' ? 'win' : process.platform;
            if (!['win', 'linux'].includes(os))
                throw new Error(`Unsupported OS: ${os}`);
            const dlurl = `https://minecraft.azureedge.net/bin-${os}/${value}`;
            return dlurl;
        });
    }
    static camelcaseToDashCase(str) {
        return str.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
    }
    static createServerProperitesData(serverProperties = BedrockServer.server_properties_default) {
        let data = BedrockServer.server_properties_data;
        for (const [key, value] of Object.entries(serverProperties)) {
            const regex = new RegExp(`%%\\{${BedrockServer.camelcaseToDashCase(key)}\\}`, 'g');
            data = data.replace(regex, value);
        }
        return data;
    }
    initServerFolder(zipdata, properties = BedrockServer.server_properties_default, files = {}) {
        if (fs_1.default.existsSync(this.folder))
            try {
                fs_1.default.rmSync(this.folder, {
                    recursive: true,
                    force: true
                });
            }
            catch (e) {
                if (e.code == 'ENOPERM') {
                    console.log('Permission denied while trying to remove old folder.');
                    process.exit(1);
                }
            }
        fs_1.default.mkdirSync(this.folder, { recursive: true });
        if (!Buffer.isBuffer(zipdata))
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                zipdata = yield (yield axios_1.default.get(zipdata, {
                    responseType: 'arraybuffer'
                })).data;
                const zip = new adm_zip_1.default(zipdata);
                const zipEntries = zip.getEntries();
                for (const entry of zipEntries) {
                    console.log(`\x1b[32m[${entry.name}] Extracting...\x1b[0m`);
                    if (entry.isDirectory) {
                        continue;
                    }
                    const entryName = entry.entryName;
                    const entryData = zip.readFile(entryName);
                    const entryPath = path_1.default.join(this.folder, entryName);
                    const entryDir = path_1.default.dirname(entryPath);
                    if (!fs_1.default.existsSync(entryDir)) {
                        fs_1.default.mkdirSync(entryDir, { recursive: true });
                    }
                    fs_1.default.writeFileSync(entryPath, entryData);
                    console.log(`\x1b[32m[${entry.name}] Extracted to ${entryPath}\x1b[0m`);
                }
                for (const [filename, data] of Object.entries(files)) {
                    fs_1.default.writeFileSync(filename, data);
                }
                this.changeServerProperties(properties);
                resolve();
            }));
        const zip = new adm_zip_1.default(zipdata);
        const zipEntries = zip.getEntries();
        for (const entry of zipEntries) {
            console.log(`\x1b[32m[${entry.name}] Extracting...\x1b[0m`);
            if (entry.isDirectory) {
                continue;
            }
            const entryName = entry.entryName;
            const entryData = zip.readFile(entryName);
            const entryPath = path_1.default.join(this.folder, entryName);
            const entryDir = path_1.default.dirname(entryPath);
            if (!fs_1.default.existsSync(entryDir)) {
                fs_1.default.mkdirSync(entryDir, { recursive: true });
            }
            fs_1.default.writeFileSync(entryPath, entryData);
            console.log(`\x1b[32m[${entry.name}] Extracted to ${entryPath}\x1b[0m`);
        }
        this.changeServerProperties(properties);
        for (const [filename, data] of Object.entries(files)) {
            fs_1.default.writeFileSync(filename, data);
        }
    }
    changeServerProperties(properties) {
        const data = BedrockServer.createServerProperitesData(properties);
        fs_1.default.writeFileSync(path_1.default.join(this.folder, "server.properties"), data);
    }
    start() {
        const proc = child_process_1.default.spawn(path_1.default.resolve(path_1.default.join(this.folder, `bedrock_server${process.platform === 'win32' ? ".exe" : ""}`)), [], {
            cwd: this.folder,
            stdio: 'pipe'
        });
        this.proc = proc;
        return proc;
    }
    stop(code = 'SIGINT') {
        this.proc.kill(code);
        return this.proc.exitCode;
    }
}
BedrockServer.server_properties_default = {
    serverName: "Bedrock Server",
    allowCheats: false,
    allowList: false,
    compressionThreshold: 1,
    difficulty: 'normal',
    contentLogFileEnabled: false,
    correctPlayerMovement: false,
    defaultPlayerPermissionLevel: 'member',
    forceGamemode: false,
    gamemode: 'survival',
    levelName: 'A bedrock world',
    levelSeed: '',
    maxPlayers: 20,
    maxThreads: 8,
    onlineMode: true,
    playerIdleTimeout: 30,
    playerMovementActionDirectionThreshold: 0.85,
    playerMovementDistanceThreshold: 0.3,
    playerMovementDurationThresholdInMs: 500,
    playerMovementScoreThreshold: 20,
    serverAuthoritativeMovement: 'server-auth',
    serverAuthoritativeBlockBreaking: false,
    serverPort: 19132,
    serverPortv6: 19133,
    texturepackRequired: false,
    tickDistance: 4,
    viewDistance: 32,
    emitServerTelemetry: false
};
BedrockServer.server_properties_data = `server-name=%%{server-name}
gamemode=%%{gamemode}
force-gamemode=%%{force-gamemode}
difficulty=%%{difficulty}
allow-cheats=%%{allow-cheats}
max-players=%%{max-players}
online-mode=%%{online-mode}
allow-list=%%{allow-list}
server-port=%%{server-port}
server-portv6=%%{server-portv6}
view-distance=%%{view-distance}
tick-distance=%%{tick-distance}
player-idle-timeout=%%{player-idle-timeout}
max-threads=%%{max-threads}
level-name=%%{level-name}
level-seed=%%{level-seed}
default-player-permission-level=%%{default-player-permission-level}
texturepack-required=%%{texturepack-required}
content-log-file-enabled=%%{content-log-file-enabled}
compression-threshold=%%{compression-threshold}
server-authoritative-movement=%%{server-authoritative-movement}
player-movement-score-threshold=%%{player-movement-score-threshold}
player-movement-action-direction-threshold=%%{player-movement-action-direction-threshold}
player-movement-distance-threshold=%%{player-movement-distance-threshold}
player-movement-duration-threshold-in-ms=%%{player-movement-duration-threshold-in-ms}
correct-player-movement=%%{correct-player-movement}
server-authoritative-block-breaking=%%{server-authoritative-block-breaking}
emit-server-telemetry=%%{emit-server-telemetry}`;
exports.default = BedrockServer;
//# sourceMappingURL=index.js.map